module HAL where

import Conduit
import Data.Default (def)
import Data.Maybe (fromMaybe)
import Data.Either (fromRight)
import Data.Text

import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)

import Servant.Client (BaseUrl(..), Scheme(..), ClientM, ClientError, runClientM, mkClientEnv)

import HAL.Client
import HAL.Doc.Corpus
import HAL.Doc.Struct

import Servant.API
import Data.Aeson

batchSize :: Int
batchSize = 1000

getMetadataWith :: Text -> Maybe Int -> Maybe Integer -> IO (Either ClientError (Response Corpus))
getMetadataWith q start rows = do
  manager' <- newManager tlsManagerSettings
  runHalAPIClient $ search (Just requestedFields) [q] Nothing start rows

getMetadataWithC :: Text -> Maybe Int -> Maybe Integer -> IO (Either ClientError (Maybe Integer, ConduitT () Corpus IO ()))
getMetadataWithC q start rows = do
  manager' <- newManager tlsManagerSettings
  -- First, estimate the total number of documents
  eRes <- runHalAPIClient $ search (Just requestedFields) [q] Nothing (Just 0) (Just 1)
  pure $ get' q start rows <$> eRes
  where
    get' :: Text -> Maybe Int -> Maybe Integer -> Response Corpus -> (Maybe Integer, ConduitT () Corpus IO ())
    get' q start rows (Response { _numFound }) =
      ( Just numResults
      , yieldMany [0..]
        .| takeC (fromInteger numPages)
        .| concatMapMC (getPage q start'))
      where
        start' = fromMaybe 0 start
        rows' = fromMaybe _numFound rows
        numResults = rows' - (fromIntegral start')
        numPages = numResults `div` (fromIntegral batchSize) + 1
      
    getPage :: Text -> Int -> Int -> IO [Corpus]
    getPage q start pageNum = do
      let offset = start + pageNum * batchSize
      eRes <- runHalAPIClient $ search (Just requestedFields) [q] Nothing (Just offset) (Just $ fromIntegral batchSize)
      pure $ case eRes of
        Left _ -> []
        Right (Response { _docs }) -> _docs
    printDoc :: Corpus -> IO Corpus
    printDoc c@(Corpus { _corpus_docid, _corpus_title }) = do
      print $ show _corpus_title
      pure c

countResults :: Text -> IO (Either ClientError Integer)
countResults q = do
  manager' <- newManager tlsManagerSettings
  -- First, estimate the total number of documents
  eRes <- runHalAPIClient $ search (Just requestedFields) [q] Nothing (Just 0) (Just 1)
  pure $ count' <$> eRes
    where
      count' :: Response Corpus -> Integer
      count' (Response { _numFound }) = _numFound
      
requestedFields :: Text
requestedFields = "docid,title_s,abstract_s,submittedDate_s,source_s,authFullName_s,authOrganism_s"

structFields :: Text
structFields = "docid,label_s,parentDocid_i"

runHalAPIClient :: (FromJSON doc, ToHttpApiData doc) =>
  ClientM (Response doc) -> IO (Either ClientError (Response doc))
runHalAPIClient cmd = do
  manager' <- newManager tlsManagerSettings
  runClientM cmd (mkClientEnv manager' $ BaseUrl Https "api.archives-ouvertes.fr" 443 "")

runStructureRequest :: Maybe Text -> IO (Either ClientError (Response Struct))
runStructureRequest rq =
  runHalAPIClient $ structure (Just structFields) rq (Just 10000)

runSearchRequest :: [Text] -> IO (Either ClientError (Response Corpus))
runSearchRequest rq =
  runHalAPIClient $ search (Just requestedFields) rq Nothing Nothing Nothing

generateRequestByStructID :: Text -> [Text] -> Text
generateRequestByStructID rq struct_ids =
  "text:" <>
  rq
  <> " AND "
  <> "structId_i:("
  <> (flattenPipe struct_ids)
  <> ")"

flattenPipe :: [Text] -> Text
flattenPipe (x:[]) = x
flattenPipe (x:xs) = x <> " || " <> flattenPipe xs
